import os
from pm4pycvxopt import util

from pm4pycvxopt.util.lp.versions import cvxopt_solver, cvxopt_solver_custom_align, cvxopt_solver_custom_align_ilp, cvxopt_solver_custom_align_arm

from pm4py.util.lp import factory


custom_solver = cvxopt_solver_custom_align
try:
    # for ARM-based Linux, we need to use a different call to GLPK
    if "arm" in str(os.uname()[-1]):
        custom_solver = cvxopt_solver
except:
    pass

factory.CVXOPT = "cvxopt"
factory.CVXOPT_SOLVER_CUSTOM_ALIGN = "cvxopt_solver_custom_align"
factory.CVXOPT_SOLVER_CUSTOM_ALIGN_ILP = "cvxopt_solver_custom_align_ilp"

factory.VERSIONS_APPLY[factory.CVXOPT] = cvxopt_solver.apply
factory.VERSIONS_GET_PRIM_OBJ[factory.CVXOPT] = cvxopt_solver.get_prim_obj_from_sol
factory.VERSIONS_GET_POINTS_FROM_SOL[factory.CVXOPT] = cvxopt_solver.get_points_from_sol

factory.VERSIONS_APPLY[factory.CVXOPT_SOLVER_CUSTOM_ALIGN] = custom_solver.apply
factory.VERSIONS_GET_PRIM_OBJ[factory.CVXOPT_SOLVER_CUSTOM_ALIGN] = custom_solver.get_prim_obj_from_sol
factory.VERSIONS_GET_POINTS_FROM_SOL[
    factory.CVXOPT_SOLVER_CUSTOM_ALIGN] = custom_solver.get_points_from_sol

factory.VERSIONS_APPLY[factory.CVXOPT_SOLVER_CUSTOM_ALIGN_ILP] = cvxopt_solver_custom_align_ilp.apply
factory.VERSIONS_GET_PRIM_OBJ[factory.CVXOPT_SOLVER_CUSTOM_ALIGN_ILP] = cvxopt_solver_custom_align_ilp.get_prim_obj_from_sol
factory.VERSIONS_GET_POINTS_FROM_SOL[
    factory.CVXOPT_SOLVER_CUSTOM_ALIGN_ILP] = cvxopt_solver_custom_align_ilp.get_points_from_sol
factory.DEFAULT_LP_SOLVER_VARIANT = factory.CVXOPT_SOLVER_CUSTOM_ALIGN

try:
    from pm4py.util.lp import solver as factory

    factory.CVXOPT = "cvxopt"
    factory.CVXOPT_SOLVER_CUSTOM_ALIGN = "cvxopt_solver_custom_align"
    factory.CVXOPT_SOLVER_CUSTOM_ALIGN_ILP = "cvxopt_solver_custom_align_ilp"

    factory.VERSIONS_APPLY[factory.CVXOPT] = cvxopt_solver.apply
    factory.VERSIONS_GET_PRIM_OBJ[factory.CVXOPT] = cvxopt_solver.get_prim_obj_from_sol
    factory.VERSIONS_GET_POINTS_FROM_SOL[factory.CVXOPT] = cvxopt_solver.get_points_from_sol

    factory.VERSIONS_APPLY[factory.CVXOPT_SOLVER_CUSTOM_ALIGN] = custom_solver.apply
    factory.VERSIONS_GET_PRIM_OBJ[factory.CVXOPT_SOLVER_CUSTOM_ALIGN] = custom_solver.get_prim_obj_from_sol
    factory.VERSIONS_GET_POINTS_FROM_SOL[
        factory.CVXOPT_SOLVER_CUSTOM_ALIGN] = custom_solver.get_points_from_sol

    factory.VERSIONS_APPLY[factory.CVXOPT_SOLVER_CUSTOM_ALIGN_ILP] = cvxopt_solver_custom_align_ilp.apply
    factory.VERSIONS_GET_PRIM_OBJ[factory.CVXOPT_SOLVER_CUSTOM_ALIGN_ILP] = cvxopt_solver_custom_align_ilp.get_prim_obj_from_sol
    factory.VERSIONS_GET_POINTS_FROM_SOL[
        factory.CVXOPT_SOLVER_CUSTOM_ALIGN_ILP] = cvxopt_solver_custom_align_ilp.get_points_from_sol
    factory.DEFAULT_LP_SOLVER_VARIANT = factory.CVXOPT_SOLVER_CUSTOM_ALIGN
except:
    pass

from pm4py.algo.conformance.alignments.versions import state_equation_a_star

state_equation_a_star.DEFAULT_LP_SOLVER_VARIANT = factory.CVXOPT_SOLVER_CUSTOM_ALIGN

from pm4py.objects.stochastic_petri import lp_perf_bounds

lp_perf_bounds.DEFAULT_LP_SOLVER_VARIANT = factory.CVXOPT

from pm4py.algo.conformance.alignments import factory as align_factory

align_factory.DEFAULT_VARIANT = align_factory.VERSION_STATE_EQUATION_A_STAR


__version__ = '0.0.9'
__doc__ = "Process Mining for Python - CVXOpt Support"
__author__ = 'PADS'
__author_email__ = 'pm4py@pads.rwth-aachen.de'
__maintainer__ = 'PADS'
__maintainer_email__ = "pm4py@pads.rwth-aachen.de"



import sys

from cvxopt import blas
from cvxopt import glpk

this_options = {}
this_options["LPX_K_MSGLEV"] = 0
this_options["msg_lev"] = "GLP_MSG_OFF"
this_options["show_progress"] = False
this_options["presolve"] = "GLP_ON"
this_options["tol_bnd"] = 10**-5
this_options["tol_piv"] = 10**-5
this_options["obj_ll"] = 10**-5
this_options["obj_ul"] = 10**-5
this_options["obj_ul"] = 10**-5


def custom_solve_lp(c, G, h, A, b):
    status, x, z, y = glpk.lp(c, G, h, A, b, options=this_options)

    if status == 'optimal':
        pcost = blas.dot(c, x)
    else:
        pcost = None

    return {'status': status, 'x': x, 'primal objective': pcost}


def apply(c, Aub, bub, Aeq, beq, parameters=None):
    """
    Gets the overall solution of the problem

    Parameters
    ------------
    c
        c parameter of the algorithm
    Aub
        A_ub parameter of the algorithm
    bub
        b_ub parameter of the algorithm
    Aeq
        A_eq parameter of the algorithm
    beq
        b_eq parameter of the algorithm
    parameters
        Possible parameters of the algorithm

    Returns
    -------------
    sol
        Solution of the LP problem by the given algorithm
    """
    sol = custom_solve_lp(c, Aub, bub, Aeq, beq)

    return sol


def get_prim_obj_from_sol(sol, parameters=None):
    """
    Gets the primal objective from the solution of the LP problem

    Parameters
    -------------
    sol
        Solution of the ILP problem by the given algorithm
    parameters
        Possible parameters of the algorithm

    Returns
    -------------
    prim_obj
        Primal objective
    """
    return sol["primal objective"]


def get_points_from_sol(sol, parameters=None):
    """
    Gets the points from the solution

    Parameters
    -------------
    sol
        Solution of the LP problem by the given algorithm
    parameters
        Possible parameters of the algorithm

    Returns
    -------------
    points
        Point of the solution
    """
    if parameters is None:
        parameters = {}

    maximize = parameters["maximize"] if "maximize" in parameters else False
    return_when_none = parameters["return_when_none"] if "return_when_none" in parameters else False
    var_corr = parameters["var_corr"] if "var_corr" in parameters else {}

    if sol and 'x' in sol and sol['x'] is not None:
        return list(sol['x'])
    else:
        if return_when_none:
            if maximize:
                return [sys.float_info.max] * len(list(var_corr.keys()))
            return [sys.float_info.min] * len(list(var_corr.keys()))



from os.path import dirname, join

from setuptools import setup

import pm4pycvxopt


def read_file(filename):
    with open(join(dirname(__file__), filename)) as f:
        return f.read()


setup(
    name=pm4pycvxopt.__name__,
    version=pm4pycvxopt.__version__,
    description=pm4pycvxopt.__doc__.strip(),
    long_description=read_file('README.md'),
    author=pm4pycvxopt.__author__,
    author_email=pm4pycvxopt.__author_email__,
    py_modules=[pm4pycvxopt.__name__],
    include_package_data=True,
    packages=['pm4pycvxopt', 'pm4pycvxopt.util', 'pm4pycvxopt.util.lp', 'pm4pycvxopt.util.lp.versions'],
    url='http://www.pm4py.org',
    license='GPL 3.0',
    install_requires=[
        'pm4py',
        'cvxopt'
    ],
    project_urls={
        'Documentation': 'http://pm4py.pads.rwth-aachen.de/documentation/',
        'Source': 'https://github.com/pm4py/pm4py-source',
        'Tracker': 'https://github.com/pm4py/pm4py-source/issues',
    }
)
